# Copyright 2021 (c) Anna Schumaker.
from . import artwork
from . import scale
from . import controls
from lib import publisher
from lib import settings
from gi.repository import GObject
from gi.repository import Gst, GLib
import tagdb

class Player(GObject.GObject):
    def __init__(self):
        GObject.GObject.__init__(self)
        self.video = Gst.ElementFactory.make("fakesink")
        self.playbin = Gst.ElementFactory.make("playbin")
        self.playbin.set_property("video-sink", self.video)
        self.Autopause = scale.AutoPauseScale()

        self.bus = self.playbin.get_bus()
        self.bus.add_signal_watch()
        self.bus.connect("message::eos", self.next)
        self.bus.connect("message::state-changed", self.on_state_changed)
        self.bus.connect("message::tag", self.on_tag)

        self.Artwork = artwork.Artwork()

        self.track = tagdb.Tracks[settings.get_int("audio.trackid")]
        self.load_set_state(self.track, Gst.State.PAUSED)
        if self.track:
            self.track.add_to_playlist("Previous")

    def duration(self):
        (res, dur) = self.playbin.query_duration(Gst.Format.TIME)
        return dur if res == True else 0

    def get_state(self):
        (ret, state, pending) = self.playbin.get_state(Gst.CLOCK_TIME_NONE)
        if ret == Gst.StateChangeReturn.SUCCESS:
            return state
        return ret

    def get_volume(self):
        return self.playbin.get_property("volume")

    def load_track(self, track):
        prev = self.track
        self.track = track
        if track is not None:
            settings.set("audio.trackid", track.trackid)
            uri = track.filepath().absolute().as_uri()
            self.playbin.set_property("uri", uri)
        self.emit("track-changed", prev, self.track)

    def load_set_state(self, track, state):
        self.playbin.set_state(Gst.State.READY)
        self.load_track(track)
        if track is not None:
            self.playbin.set_state(state)

    def next(self, *args):
        duration = self.duration()
        if duration > 0 and (self.runtime() / duration) > (2 / 3):
            self.track.played()

        (track, cont) = tagdb.Stack.next()
        state = Gst.State.PLAYING if cont else Gst.State.PAUSED
        self.load_set_state(track, state)

    def on_state_changed(self, bus, message):
        (old, new, pending) = message.parse_state_changed()
        self.emit("state-changed", old, new, pending)

    def on_tag(self, bus, message):
        taglist = message.parse_tag()
        (res, sample) = taglist.get_sample("image")
        if res == True:
            self.Artwork.set_from_sample(sample)
        else:
            self.Artwork.reset()

    def pause(self, *args):
        self.playbin.set_state(Gst.State.PAUSED)

    def play(self, *args):
        self.playbin.set_state(Gst.State.PLAYING)

    def playpause(self, *args):
        if self.get_state() == Gst.State.PLAYING:
            self.pause()
        else:
            self.play()

    def play_track(self, track):
        if track == self.track:
            return False
        self.load_set_state(track, Gst.State.PLAYING)
        return True

    def position(self):
        (res, pos) = self.playbin.query_position(Gst.Format.TIME)
        return pos if res == True else 0

    def previous(self, *args):
        self.play_track(tagdb.Stack.previous())

    def runtime(self):
        if self.playbin.clock == None:
            return 0
        return self.playbin.clock.get_time() - self.playbin.base_time

    def seek(self, value):
        self.playbin.seek_simple(Gst.Format.TIME, Gst.SeekFlags.FLUSH, value)

    def set_volume(self, volume):
        self.playbin.set_property("volume", volume)

    @GObject.Signal(arg_types=(Gst.State, Gst.State, Gst.State))
    def state_changed(self, old, new, pending):
        pass

    @GObject.Signal(arg_types=(tagdb.track.Track, tagdb.track.Track))
    def track_changed(self, prev, new):
        pass
